==> wallets-0.0.1.tgz: package/extension_wallet.ts <==
import { Account, Caller, Chain, ContractBuilder, Errors, Receipt, TxResult, Wallet } from "./types";
import { encodeFunction } from "./tx_helper";
import { defaultAbiCoder as AbiCoder } from "@ethersproject/abi";

type CPChainMessagePayload = {
  status: "not_unlocked" | "allowed" | "rejected";
  address?: string;
  tx_hash?: string;
  chain?: Chain;
};

type CPChainMessage = {
  type: "FROM_CPCHAIN";
  action: "connect" | "submit-tx" | "add-token";
  payload: CPChainMessagePayload;
};

type SubmitTxRepl = {
  status: 'allowed' | 'rejected',
  tx_hash?: string
}

function submitTx (payload: { to: string, data: string, value: number }): Promise<SubmitTxRepl> {
  return new Promise<SubmitTxRepl>((resolve, reject) => {
    const listener = (event: any) => {
      if (event.data.type && (event.data.type === 'FROM_CPCHAIN')) {
        if (event.data.action === 'submit-tx') {
          const data = event.data as CPChainMessage
          window.removeEventListener('message', listener)
          resolve(data.payload as any)
        }
      }
    }
    window.addEventListener('message', listener)
    window.postMessage({
      type: 'TO_CPCHAIN',
      action: 'submit-tx',
      payload
    })
  })
}

function extensionWalletExists(): Promise<boolean> {
  return new Promise<boolean>((resolve, _reject) => {
    setTimeout(() => {
      resolve(!!document.getElementById("__cpchain_extension_wallet_checker"));
    }, 100);
  });
}

function connectWallet(): Promise<CPChainMessagePayload> {
  return new Promise<CPChainMessagePayload>((resolve, reject) => {
    const listener = (event: any) => {
      if (event.data.type && event.data.type === "FROM_CPCHAIN") {
        if (event.data.action === "connect") {
          const data = event.data as CPChainMessage;
          window.removeEventListener("message", listener);
          resolve(data.payload);
        }
      }
    };
    window.addEventListener("message", listener);
    window.postMessage({ type: "TO_CPCHAIN", action: "connect" }, "*");
  });
}

class ExtensionTxResult implements TxResult {
  hash: string;
  constructor(hash: string) {
    this.hash = hash;
  }
  wait(): Promise<Receipt> {
    throw new Error("Method not implemented.");
  }
}

class ExtensionCaller implements Caller {
  _value: number
  _gas: number
  constructor(private method: ABIMethod, private to: string, private data: string) {
    this._value = 0
    this._gas = 25000
  }
  async view(): Promise<unknown> {
    const res = await fetch("https://civilian.cpchain.io", {
      method: 'POST',
      headers: {
        'content-type': 'application/json'
      },
      body: JSON.stringify({
        jsonrpc: '2.0',
        method: 'eth_call',
        id: '1',
        params: [
          {
            to: this.to,
            data: this.data
          },
          'latest'
        ]
      })
    })
    const r = await res.json()
    const outputs = this.method.outputs.map(i => i.type)
    const results = AbiCoder.decode(outputs, r.result)
    return results
  }
  gasLimit(gas: number): Caller {
    this._gas = gas;
    return this 
  }
  value(value: number): Caller {
    this._value = value
    return this
  }
  async submit(): Promise<TxResult> {
    const tx = await submitTx({
      to: this.to,
      data: this.data,
      value: this._value
    })
    if (tx.status === 'allowed' && tx.tx_hash) {
      return new ExtensionTxResult(tx.tx_hash)
    }
    throw new Error("Rejected")
  }
}

type ABIMethod = {
  name: string
  inputs: { indexed: boolean, name: string, type: string }[]
  outputs: { name: string, type: string }[]
  payable: boolean
  stateMutability: 'payable' | 'nonpayable' | 'view'
  type: 'event' | 'function' | 'constructor'
}

class ExtensionContractBuilder implements ContractBuilder {
  _abi: ABIMethod[]
  constructor(abi: string, private address: string) {
    this._abi = JSON.parse(abi)
  }
  call(methodName: string, ...args: any): Caller {
    for (const m of this._abi) {
      if (m.name === methodName) {
        // 将合约调用转化为 bytes
        const data = encodeFunction(m.name, {
          types: m.inputs.map(i => i.type),
          values: args
        })
        return new ExtensionCaller(m, this.address, data)
      }
    }
    throw new Error(`Method ${methodName} not found in your ABI`);
  }
}

export class CPChainExtensionWallet implements Wallet {
  connected: boolean;
  chain?: Chain
  constructor() {
    this.connected = false;
  }
  contract(abi: string, contract_address: string): ContractBuilder {
    // if (!this.connected || !this.chain) {
    //   throw new Error('Wallet not connected')
    // }
    return new ExtensionContractBuilder(abi, contract_address)
  }
  connect(): Promise<Account> {
    this.connected = false
    return extensionWalletExists()
      .then((exists) => {
        if (!exists) {
          throw new Error(Errors.NotFoundExtensionWallet);
        }
        return connectWallet();
      })
      .then((data) => {
        this.connected = true;
        this.chain = data.chain
        return { address: data.address, chain: data.chain } as Account;
      });
  }
}

==> wallets-0.0.2.tgz: package/extension_wallet.ts <==
import { Account, Caller, Chain, ContractBuilder, Errors, Receipt, TxResult, Wallet } from "./types";
import { encodeFunction } from "./tx_helper";
import { defaultAbiCoder as AbiCoder } from "@ethersproject/abi";

export type CPChainMessagePayload = {
  status: "not_unlocked" | "allowed" | "rejected";
  address?: string;
  tx_hash?: string;
  chain?: Chain;
};

export type CPChainMessage = {
  type: "FROM_CPCHAIN";
  action: "connect" | "submit-tx" | "add-token";
  payload: CPChainMessagePayload;
};

export type SubmitTxRepl = {
  status: 'allowed' | 'rejected',
  tx_hash?: string
}

export function submitTx (payload: { to: string, data: string, value: number }): Promise<SubmitTxRepl> {
  return new Promise<SubmitTxRepl>((resolve, reject) => {
    const listener = (event: any) => {
      if (event.data.type && (event.data.type === 'FROM_CPCHAIN')) {
        if (event.data.action === 'submit-tx') {
          const data = event.data as CPChainMessage
          window.removeEventListener('message', listener)
          resolve(data.payload as any)
        }
      }
    }
    window.addEventListener('message', listener)
    window.postMessage({
      type: 'TO_CPCHAIN',
      action: 'submit-tx',
      payload
    })
  })
}

export function extensionWalletExists(): Promise<boolean> {
  return new Promise<boolean>((resolve, _reject) => {
    setTimeout(() => {
      resolve(!!document.getElementById("__cpchain_extension_wallet_checker"));
    }, 100);
  });
}

export function connectWallet(): Promise<CPChainMessagePayload> {
  return new Promise<CPChainMessagePayload>((resolve, reject) => {
    const listener = (event: any) => {
      if (event.data.type && event.data.type === "FROM_CPCHAIN") {
        if (event.data.action === "connect") {
          const data = event.data as CPChainMessage;
          window.removeEventListener("message", listener);
          resolve(data.payload);
        }
      }
    };
    window.addEventListener("message", listener);
    window.postMessage({ type: "TO_CPCHAIN", action: "connect" }, "*");
  });
}

export class ExtensionTxResult implements TxResult {
  hash: string;
  constructor(hash: string) {
    this.hash = hash;
  }
  wait(): Promise<Receipt> {
    throw new Error("Method not implemented.");
  }
}

export class ExtensionCaller implements Caller {
  _value: number
  _gas: number
  constructor(private method: ABIMethod, private to: string, private data: string) {
    this._value = 0
    this._gas = 25000
  }
  async view(): Promise<unknown> {
    const res = await fetch("https://civilian.cpchain.io", {
      method: 'POST',
      headers: {
        'content-type': 'application/json'
      },
      body: JSON.stringify({
        jsonrpc: '2.0',
        method: 'eth_call',
        id: '1',
        params: [
          {
            to: this.to,
            data: this.data
          },
          'latest'
        ]
      })
    })
    const r = await res.json()
    const outputs = this.method.outputs.map(i => i.type)
    const results = AbiCoder.decode(outputs, r.result)
    return results
  }
  gasLimit(gas: number): Caller {
    this._gas = gas;
    return this 
  }
  value(value: number): Caller {
    this._value = value
    return this
  }
  async submit(): Promise<TxResult> {
    const tx = await submitTx({
      to: this.to,
      data: this.data,
      value: this._value
    })
    if (tx.status === 'allowed' && tx.tx_hash) {
      return new ExtensionTxResult(tx.tx_hash)
    }
    throw new Error("Rejected")
  }
}

export type ABIMethod = {
  name: string
  inputs: { indexed: boolean, name: string, type: string }[]
  outputs: { name: string, type: string }[]
  payable: boolean
  stateMutability: 'payable' | 'nonpayable' | 'view'
  type: 'event' | 'function' | 'constructor'
}

export class ExtensionContractBuilder implements ContractBuilder {
  _abi: ABIMethod[]
  constructor(abi: string, private address: string) {
    this._abi = JSON.parse(abi)
  }
  call(methodName: string, ...args: any): Caller {
    for (const m of this._abi) {
      if (m.name === methodName) {
        // 将合约调用转化为 bytes
        const data = encodeFunction(m.name, {
          types: m.inputs.map(i => i.type),
          values: args
        })
        return new ExtensionCaller(m, this.address, data)
      }
    }
    throw new Error(`Method ${methodName} not found in your ABI`);
  }
}

export class CPChainExtensionWallet implements Wallet {
  connected: boolean;
  chain?: Chain
  constructor() {
    this.connected = false;
  }
  contract(abi: string, contract_address: string): ContractBuilder {
    // if (!this.connected || !this.chain) {
    //   throw new Error('Wallet not connected')
    // }
    return new ExtensionContractBuilder(abi, contract_address)
  }
  connect(): Promise<Account> {
    this.connected = false
    return extensionWalletExists()
      .then((exists) => {
        if (!exists) {
          throw new Error(Errors.NotFoundExtensionWallet);
        }
        return connectWallet();
      })
      .then((data) => {
        this.connected = true;
        this.chain = data.chain
        return { address: data.address, chain: data.chain } as Account;
      });
  }
}
